"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebhookService = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const rest_1 = require("@loopback/rest");
const repositories_1 = require("../repositories");
const api_key_service_1 = require("./api-key.service");
const services_1 = require("../../../services");
let WebhookService = class WebhookService {
    constructor(webhookSubscriptionRepo, apiKeyService, httpClientService) {
        this.webhookSubscriptionRepo = webhookSubscriptionRepo;
        this.apiKeyService = apiKeyService;
        this.httpClientService = httpClientService;
    }
    async addToSubscription(url, key) {
        try {
            const webhookSubscriptions = await this.webhookSubscriptionRepo.find({
                where: {
                    url,
                    key,
                },
            });
            if (webhookSubscriptions.length === 0) {
                await this.webhookSubscriptionRepo.create({
                    key,
                    url,
                });
            }
        }
        catch (error) {
            throw new rest_1.HttpErrors.InternalServerError('Could not add to subscription');
        }
    }
    async getUrlOfSubscritption(key) {
        const webhookSubscriptions = await this.webhookSubscriptionRepo.find({
            where: {
                key,
            },
        });
        if (webhookSubscriptions.length === 1) {
            return webhookSubscriptions[0].url;
        }
        return null;
    }
    async triggerWebhook(event) {
        try {
            const subscribers = await this.webhookSubscriptionRepo.find({
                where: { key: event.key },
            });
            if (subscribers.length > 0) {
                const timestamp = Date.now();
                const signature = await this.apiKeyService.generateSignature(event, timestamp);
                const postPromises = subscribers.map(subscriber => this.httpClientService.post(subscriber.url, event, {
                    headers: {
                        'content-type': 'application/json',
                        'x-task-signature': signature,
                        'x-task-timestamp': timestamp.toString(),
                    },
                }));
                await Promise.all(postPromises);
            }
        }
        catch (error) {
            throw new rest_1.HttpErrors.InternalServerError('could not trigger webhook');
        }
    }
};
exports.WebhookService = WebhookService;
exports.WebhookService = WebhookService = tslib_1.__decorate([
    (0, core_1.injectable)({ scope: core_1.BindingScope.SINGLETON }),
    tslib_1.__param(0, (0, repository_1.repository)(repositories_1.WebhookSubscriptionRepository)),
    tslib_1.__param(1, (0, core_1.service)(api_key_service_1.ApiKeyService)),
    tslib_1.__param(2, (0, core_1.service)(services_1.HttpClientService)),
    tslib_1.__metadata("design:paramtypes", [repositories_1.WebhookSubscriptionRepository,
        api_key_service_1.ApiKeyService,
        services_1.HttpClientService])
], WebhookService);
//# sourceMappingURL=webhook.service.js.map