"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserTaskService = void 0;
const tslib_1 = require("tslib");
const core_1 = require("@loopback/core");
const repository_1 = require("@loopback/repository");
const keys_1 = require("../keys");
const repositories_1 = require("../repositories");
const types_1 = require("../types");
const camunda_service_1 = require("./camunda.service");
let UserTaskService = class UserTaskService {
    constructor(taskRepo, userTaskRepo, outgoing, camundaService) {
        this.taskRepo = taskRepo;
        this.userTaskRepo = userTaskRepo;
        this.outgoing = outgoing;
        this.camundaService = camundaService;
    }
    async complete(taskId, userTaskId) {
        const task = await this.taskRepo.findById(taskId);
        const userTask = await this.userTaskRepo.findById(userTaskId);
        await this.camundaService.completeUserTask(userTask.externalId);
        await this.userTaskRepo.updateById(userTaskId, {
            status: types_1.UserTaskStatus.Completed,
        });
        if (task.id && task.externalId) {
            await this.updateList(task.id, task.externalId);
        }
        await this.outgoing.publish({
            key: types_1.EventType.UserTaskCompleted,
            payload: {
                task,
                userTask,
            },
            timestamp: Date.now(),
            source: types_1.Source.TaskService,
        });
    }
    async updateList(taskId, instanceId) {
        const pendingTasks = await this.camundaService.getPendingUserTasks(instanceId);
        if (pendingTasks.length > 0) {
            await this.userTaskRepo.createAll(pendingTasks.map(camundaTask => ({
                taskId,
                status: types_1.UserTaskStatus.Pending,
                externalId: camundaTask.id,
                name: camundaTask.name,
            })));
        }
    }
};
exports.UserTaskService = UserTaskService;
exports.UserTaskService = UserTaskService = tslib_1.__decorate([
    tslib_1.__param(0, (0, repository_1.repository)(repositories_1.TaskRepository)),
    tslib_1.__param(1, (0, repository_1.repository)(repositories_1.UserTaskRepository)),
    tslib_1.__param(2, (0, core_1.inject)(keys_1.TaskServiceBindings.OUTGOING_CONNECTOR)),
    tslib_1.__param(3, (0, core_1.service)(camunda_service_1.CamundaService)),
    tslib_1.__metadata("design:paramtypes", [repositories_1.TaskRepository,
        repositories_1.UserTaskRepository, Object, camunda_service_1.CamundaService])
], UserTaskService);
//# sourceMappingURL=user-task.service.js.map